from Globals                import DTMLFile
from OFS.Folder             import Folder
from OFS.DTMLMethod         import DTMLMethod
from Products.OrderedFolder import SubObjectTypeSupport 
from Products.OFSP.Version  import Version

from Products.CoreSessionTracking.SessionDataManager import constructSessionDataManager
from Products.CoreSessionTracking.SessionIdManager   import constructSessionIdManager

from Products.OrderedFolder import OrderedFolder

import WizardPageFolder, SmartWizardDTML


manage_addSmartWizardForm = DTMLFile('dtml/manage_addSmartWizardForm', globals())


def manage_addSmartWizard(self, id, title='', REQUEST=None):
    '''manage_addSmartWizard(self, id, title='', REQUEST=None) --> add a SmartWizard; return None or HTML'''

    wizard = SmartWizard(id, title)
    self._setObject(wizard.id, wizard)

    wizard = getattr(self, wizard.id)
    # Create a Core Session Tracking object; it must be here, since the DataManager gives me
    # grief, when trying to initialize it in the SmartWizard.__init__
    constructSessionDataManager(wizard, 'wizardSession', 'Manages Session Data', timeout_mins=240)
    # If there is already a session Id Manager in the path, then we cannot create another
    # one. Makes no sense to me, but oh well.
    try:
        constructSessionIdManager(wizard, 'session_id_mgr', 'Manages Session Ids of User') 
    except:
        pass

    if REQUEST:
        return self.manage_main(self, REQUEST, update_menu=1)



class SmartWizard(SubObjectTypeSupport.SubObjectTypeSupport, Folder):
    '''SmartWizard class - Represents one specific Wizard that can be executed.'''

    meta_type = 'Smart Wizard'
    icon = 'misc_/SmartWizard/Wizard'

    _properties = ( {'id': 'title', 'type': 'string', 'mode': 'w', 'default': ''},
                    {'id': 'wizardWidth', 'type': 'string', 'mode': 'w', 'default': '100%'}, )

    manage_options = Folder.manage_options + SubObjectTypeSupport.SubObjectTypeSupport.manage_options

    selected_meta_types = ('DTML Method', 'Folder', 'Transparent Folder', 'Ordered Folder',
                           'Wizard Page Folder', 'Image', 'Session Id Manager',
                           'Session Data Manager', 'Script (Python)', 'File')
    

    def __init__(self, id, title=''):
        '''__init__(self, id, title='') --> initialize the object; return None'''
        
        # initialize the properties, otherwise the 'Properties' tab will spit at you.
        # Seez, when will they fix that?
        for property in self._properties:
            setattr(self, property['id'], property['default'])

        self.id = str(id)
        self.title = title

        # Create the folder that contains the pages
        pages = WizardPageFolder.WizardPageFolder()
        pages.id = 'Pages'
        self._setObject('Pages', pages)

        # Create a Folder that contains all the versions
        versions = OrderedFolder.OrderedFolder()
        versions.selected_meta_types = ('Version',)
        versions.id = 'Versions'
        self._setObject('Versions', versions)

        # Create the initial Header, Footer and CSS Style Sheet
        header = DTMLMethod(SmartWizardDTML.defaultWizardHeader, __name__ = 'wizardHeader')
        self._setObject('wizardHeader', header)
        footer = DTMLMethod(SmartWizardDTML.defaultWizardFooter, __name__ = 'wizardFooter')
        self._setObject('wizardFooter', footer)
        css = DTMLMethod(SmartWizardDTML.defaultWizardCSS, __name__ = 'wizardCSS')
        self._setObject('wizardCSS', css)

        # Create logo method
        logo = DTMLMethod(SmartWizardDTML.defaultWizardLogo, __name__ = 'wizardLogo')
        self._setObject('wizardLogo', logo)        

        # Create Overview method
        overview = DTMLMethod(SmartWizardDTML.defaultWizardOverview, __name__ = 'wizardOverview')
        self._setObject('wizardOverview', overview)        

        # Create initial Status method
        nav = DTMLMethod(SmartWizardDTML.defaultWizardStatus, __name__ = 'wizardStatus')
        self._setObject('wizardStatus', nav)        

        # Create navigation bar method
        nav = DTMLMethod(SmartWizardDTML.defaultWizardNavigationBar, __name__ = 'wizardNavigationBar')
        self._setObject('wizardNavigationBar', nav)        
    
        # Create initial Description
        desc = DTMLMethod(SmartWizardDTML.defaultWizardDescription, __name__ = 'wizardDescription')
        self._setObject('wizardDescription', desc)        

        # Create index_html
        index = DTMLMethod(SmartWizardDTML.defaultWizardIndex, __name__ = 'index_html')
        self._setObject('index_html', index)        
        
        # Create initial main window
        main = DTMLMethod(SmartWizardDTML.defaultWizardMainWindow, __name__ = 'wizardMainWindow')
        self._setObject('wizardMainWindow', main)        


    def initializeSession(self):
        '''initializeSession(self) --> add the session daat into the REQUEST, so we can use it; return None'''
        data = self.wizardSession.getSessionData()
        if not data.has_key('activePageIndex'):
            data.set('activePageIndex', 0)
        self.REQUEST.set('session', data)


    def getCurrentPageIndex(self):
        '''getCurrentPageIndex(self) --> return the index of the active (current) page; return int'''
        return self.REQUEST['session']['activePageIndex']
    

    def changeActivePageIndex(self, index):
        '''changeActivePageIndex(self, index) --> Change the active page index after checking, if it is
           in the acceptable range; return None'''
        if index >= 0 and index < len(self.Pages.objectIds()):
            self.REQUEST['session'].set('activePageIndex', index)
            
            
    def getCurrentPage(self):
        '''getCurrentPage(self) --> get the actual page object of the current location; return WizardPage'''
        return self.Pages.objectValues()[self.getCurrentPageIndex()]
    

    def goToNextPage(self):
        '''goToNextPage(self) --> go to the next page; return None'''
        if not self.REQUEST.has_key('session'):
            self.initializeSession()
        self.changeActivePageIndex(self.getCurrentPageIndex()+1)
        self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/index_html')


    def goToPreviousPage(self):
        '''goToPreviousPage(self) --> go to the previous page; return None'''
        if not self.REQUEST.has_key('session'):
            self.initializeSession()
        self.changeActivePageIndex(self.getCurrentPageIndex()-1)
        self.REQUEST.RESPONSE.redirect(self.absolute_url()+'/index_html')
